/*
** CONEXIONES.C
**
**
** (c) 1.997-99 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 20/May/97 Versin 1.0
**           Primera versin de este CGI.
**
** 20/May/97 Versin 1.1
**           Permite monitorizar las conexiones de cualquier usuario
**           sin ms que escribir al final un ?clave*login (* es comodn)
**
** 23/Jun/97 Versin 1.2
**           Pone la misma apariencia tpica que el resto de pginas de ARGO.
**
** 26/Jun/97 Versin 1.3
**           Aadidos CSIVs de Bilbao y Sevilla.
**
** 19/Sep/97 Versin 1.5
**           Aadido trfico acumulado Entrada/Salida, en paquetes y bytes.
**           Las centrales analgicas se identifican como tales.
**           Proporciona la IP actual de un usuario conectado.
**           Proporciona el LOGIN de una IP determinada conectada ahora.
**
** 19/Sep/97 Versin 1.51
**           Los datos de los CSIVs se leen en orden, sin comprobar las
**           fechas. Por tanto, si un usuario entra un da por un CSIV
**           ledo ms tarde que su CSIV normal, aparecer siempre como
**           STOP. Hay que contrastar con las fechas de los registros.
**           Como el escaneo se realiza sobre un nico mes, no es preciso
**           Intercambiar MES y DIA en el campo FECHA de cada REGISTRO (y ao).
**
** 10/Dic/97 Versin 1.53
**           Se incluye un mensaje para aquellos usuarios propios que
**           entran a travs de una red externa.
**           Si pongo clave correcta, me deja acceder AUNQUE no est en la red ARGO.
**           La informacin sobre IP actual se est disponible en la parte
**           inferior de la pgina *AUNQUE* no se indique clave de acceso.
**
** 11/Dic/97 Versin 2.0
**           Indica la causa de la ltima desconexin.
**           ATENCION: A partir de ahora este script es llamado tambin
**           mensualmente por un proceso *fuera* del servidor, para sacar las
**           estadsticas de cada usuario mes a mes. Para que ello sea posible
**           se permite llamar directamente al CGI desde la linea de comando,
**           y marca el principio y el final de la zona interesante con un
**           "<!mensual>" en cada linea para poder reconocerlo bien con un
**           simple awk.
**           Las estad&iacute;sticas antiguas se leen haciendo que el
**           ultimo carcter del "query-string" sea un "*".
**           Se aade un enlace a las estadsticas antiguas en la propia pgina
**           Se aade "91307" a los nmeros "de prueba".
**
** 16/Ene/98 Versin 2.1
**           Aadida "Content-Type: text/html".
**
** 06/May/98 Versin 2.2
**           Desde el 4 de Abril infova manda el "9" inicial.
**
** 19/May/98 Versin 2.3
**           Migracin de 194.235.99.* a 195.53.30.*
**
** 28/May/98 Version 2.4
**           Indica la fecha de la ultima conexion.
**
** 28/Sep/98 Version 2.5
**           Debido a que el RADIUS parece grabar algunos registros duplicados
**           en el "accounting", lo verifica y los descarta si es necesario.
**           Anade nuevos numeros en el mensaje de "numeros excluidos" para las
**           llamadas realizadas por nosotros para comprobar el sistema.
**
** 30/Sep/98 Version 2.52
**           Aunque la mayoria de los datos son correctos, la causa de la ultima
**           conexion se leia del ultimo registro leido, no de la ultima conexion
**           efectuada. No coinciden si el usuario tiene entradas por diferentes
**           CSIVs.
**           Anadida la causa de desconexion 20
**
** 13/Oct/98 Version 2.6
**           Los datos de configuracion especificos de cada ISP se especifican
**           a traves de variables "extern": clase C, clave, ISP, CSIVs, archivo historico.
**           Todas las URLs se ponen absolutas.
**           Anadido mensaje exigido por la licencia.
**
** 15/Oct/98 Version 2.7
**           Ya no es necesario hacer un include directo de "lee_log.c", ya que
**           todo se ha centralizado en el "make" y en las librerias y objetos.
**
** 20/Oct/98 El uso de una libreria de cadenas propia mejora el rendimiento.
**
** 26/Oct/98 El programa y ano genera un CORE si la entrada CLI es incorrecta.
**
** 27/Oct/98 El programa funciona aunque no se usen "@" en el username.
**           El array de accounting define el sufijo a anadir y las
**           rutinas a usar para acceder a cada fichero.
**
** 18/Nov/98 La linea de comandos permite limitar el tiempo a considerar
**           anadiendo parametros en formato yyyymmddhhmm
**
** 20/Nov/98 Anadidos los marcos horarios por defecto
**           a la hora de leer un fichero de log.
**
** 23/Nov/98 Todos los tiempos en los registros son segundos.
**           Anadidas las nuevas causas de desconexion.
**
** 25/Nov/98 La generacion de la salida HTML se
**           ha movido a un modulo externo.
**
** 01/Dic/98 Corregido un problema de filtrado por fechas
**           cuando en el log hay entradas que estan fuera
**           del intervalo temporal a considerar.
*/

#include "conexiones.h"
#include "salida_html.h"
#include <stdlib.h>
#include <stdio.h>
#include <pwd.h>
#include <ctype.h>
#include <string.h>
#include "lee_log.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define strchr(a,b)	strchr_conexiones(a,b)
#define strcmp(a,b)	strcmp_conexiones(a,b)
#include "util.c"


struct caller_id {
	char numero[CAMPO_LONG];
	long num_conexiones;
	long tiempo;
	struct caller_id *siguiente;
};

static struct datos {
	long num_conexiones;
	long tiempo;
	long long bytes_tx;
	long long bytes_rx;
	long long paquetes_tx;
	long long paquetes_rx;
	struct caller_id *numero;
} val;

static char ip_log[CAMPO_LONG];
static long fecha,fecha2;
static long ultima_conexion;
static long ultimo_tiempo;
static int causa_desconexion;

#define MAX_TIEMPO	0x7fffffff
static time_t tiempo_inicial=0;
static time_t tiempo_final=MAX_TIEMPO;

/****
static time_t convierte_tiempo_radius(char *tiempo)
{
struct tm tm;

  strptime(tiempo,"%d-%m-%Y %H:%M:%S",&tm);
  return mktime(&tm);
}
*****/

static struct datos analiza_conexion(char *f_detalle,char *nombre,t_log *log)
{
registro_log reg;
log_estado estado;
struct caller_id *numero;
char elimina_duplicado[CAMPO_LONG];
	
	elimina_duplicado[0]='\0';

	log->abre_log(f_detalle,nombre);
	while (!0) {
		estado=log->lee_registro(&reg);
		if(estado!=LOG_OK) break;
                fecha2=reg.tiempo;

                if(tiempo_inicial>fecha2) continue;
                if(tiempo_final<fecha2-reg.duracion) continue;

		if(strcmp(reg.nombre,nombre)) continue;	/* Nos lo saltamos */
		if(fecha2>fecha) {
			fecha=fecha2;
			strcpy(ip_log,reg.ip);	/* IP de esta conexin */
		}

		if(reg.tipo!=STOP) continue;	/* Nos lo saltamos */

                if(!strcmp(reg.sesion_id,elimina_duplicado)) continue;
		strcpy(elimina_duplicado,reg.sesion_id);

		if(fecha2>ultima_conexion) {
			ultima_conexion=fecha2;
			ultimo_tiempo=reg.duracion;
                        causa_desconexion=reg.causa;
		}

		if(fecha2==fecha) {
			*ip_log='\0';	/* Este IF es por esta LINEA */
		}
		val.num_conexiones++;
		val.tiempo+=reg.duracion;
		val.bytes_tx+=reg.bytes_tx;
		val.bytes_rx+=reg.bytes_rx;
		val.paquetes_tx+=reg.paquetes_tx;
		val.paquetes_rx+=reg.paquetes_rx;
		numero=val.numero;
		while(numero!=NULL) {
			if(!strcmp(numero->numero,reg.caller_id)) {
				numero->num_conexiones++;
				numero->tiempo+=reg.duracion;
				break;	/* Ya lo tenemos */
			}
			/* No lo ha encontrado an */
			numero=numero->siguiente;
		}
		if(numero!=NULL) continue;	/* Ya lo tenemos */
		numero=val.numero;	/* Lo aade */
		val.numero=malloc(sizeof(struct caller_id));
		strcpy(val.numero->numero,reg.caller_id);
		val.numero->siguiente=numero;
		val.numero->num_conexiones=1;
		val.numero->tiempo=reg.duracion;
	}
	log->cierra_log();
	return val;
}

static registro_log buscar_ip(char *f_detalle,char *ip,t_log *log)
{
registro_log reg,reg2;
log_estado estado;

	reg.tipo=STOP;
	log->abre_log(f_detalle,"");
	while (!0) {
		estado=log->lee_registro(&reg2);
		if(estado!=LOG_OK) break;
		if(strcmp(reg2.ip,ip)) continue;	/* Nos lo saltamos */
		fecha2=reg2.tiempo;
		if(fecha2<fecha) continue;/* Nos lo saltamos */
		fecha=fecha2;
		reg=reg2;
	}
	log->cierra_log();
	return reg;
}

static void lista_antiguo(char *usuario)
{
int f;
char buf[1024];

	strcpy(buf,D_HISTORICO);
	strcat(buf,usuario);
	f=open(buf,O_RDONLY);
	if(f==-1) {
		printf("<center><h1>No se encuentra informaci&oacute;n previa \n"
			"<br>sobre ese usuario</h1>\n"
			"Ello es normal si se trata de un usuario con\n"
			"<br>una antig&#252;edad inferior a un mes\n"
			"</center>");
	} else {
		while(read(f,buf,1)==1) printf("%c",buf[0]);
		close(f);
	}
	envia_4();
	envia_post();
}

/*
** El tiempo convertido es local, no UTC
** Parece ignorar los "daylight saving"
*/
time_t str2tiempo(char *tiempo)
{
struct tm tm;

  strptime(tiempo,"%Y%m%d%H%M",&tm);
  return mktime(&tm);
}

void tiempo2str(char *buf,time_t tiempo)
{
struct tm *tm;

  tm=localtime(&tiempo);
  strftime(buf,CAMPO_LONG,"%Y%m%d%H%M",tm);
}

void fija_tiempo_inicial(time_t tiempo)
{
  tiempo_inicial=tiempo;
}

void fija_tiempo_final(time_t tiempo)
{
  tiempo_final=tiempo;
}

int main_conexiones(int argc,char *argv[],char *env[])
{
char *ip;
char *var,*string;
char *clave=NULL;
char *p;
int clave_flag=0;
int anteriores_flag=0;
int contador;

	fecha=0;
	ultima_conexion=0;
	ultimo_tiempo=0;

	envia_pre();
	
	string=getenv("QUERY_STRING");
	if((string!=NULL)&&(!strncmp(string,CLAVE,strlen(CLAVE)))&&
		(strlen(string)>strlen(CLAVE)+1)&&
		(strlen(string)<CAMPO_LONG)) { /* El ltimo check evita exploits */
		clave_flag=!0;
		clave=string+strlen(CLAVE)+1;	/* Nos saltamos el separador */
	}
	if((string!=NULL)&&(strlen(string))&&(*(string+strlen(string)-1)=='*'))
		anteriores_flag=!0;
	
	ip=getenv("REMOTE_ADDR");
	if(ip!=NULL) printf("%s<p>\n",ip);	/* Solo ocurre si es llamada directa */
	else {	/* Llamada directa */
		printf("Informaci&oacute;n no disponible<p>\n");
                if(argc<2) return 0; /* Si llamada directa -> al menos un parametro */
		clave_flag=!0;
		clave=argv[1];
		if(argc>2) tiempo_inicial=str2tiempo(argv[2]);
                if(argc>3) tiempo_final=str2tiempo(argv[3]);
	}
	
	var=getenv("REMOTE_HOST");
	if(var!=NULL) printf("%s<p>\n",var);
	else printf("Informaci&oacute;n no disponible<p>\n");

	var=getenv("HTTP_USER_AGENT");
	if(var!=NULL) printf("%s<p>\n",var);
	else printf("Informaci&oacute;n no disponible<p>\n");

	envia_2();
	
	/*
	** El hack funciona desde cualquier red.
	** Las llamadas directas SIEMPRE tienen que poner la clave.
	*/
        for(contador=0;(ip!=NULL)&&(RED[contador]!=NULL);contador++) {
          if(!strncmp(ip,RED[contador],strlen(RED[contador]))) {
            contador=-1;
            break;
          }
        }

	/*
	** Otro hack para poder asignar usuarios fijos a
	** IPs fijas sin necesidad de darlos de alta en dos
	** estructuras diferentes (en IP_USUARIO y en RED).
	*/
	if(contador>=0) for(contador=0;(ip!=NULL)&&(IP_USUARIO[contador].ip!=NULL);contador++) {
	  if(!strcmp(ip,IP_USUARIO[contador].ip)) {
	    contador=-1;
	    break;
	  }
	}

	if((clave_flag)||(contador<0)) {
	registro_log reg;
	
	/* Busca el propietario de esta conexin */
	
		if(clave_flag) {	/* Pregunta genrica */
			if(anteriores_flag) {	/* meses previos */
			char *p;

				*(clave+strlen(clave)-1)='\0';
				p=clave;
				while(*p) {	/* usuario vlido */
					if((!isalnum(*p))&&(*p!='_')&&(*p!='-')) {
						exit(-1);
					}
					p++;
				}
				lista_antiguo(clave);
				exit(0);
			}
                        reg.tipo=START;
                        strcpy(reg.nombre,clave);
			if((*clave>='0')&&(*clave<='9')) {	/* Busca IP */
				ip=clave;
                                contador=0;
                                do {
                                  reg=buscar_ip(accounting[contador].path,ip,accounting[contador].log);
                                  contador++;
                                } while((reg.tipo==STOP)&&(accounting[contador].path!=NULL));
			}
		} else { /* Pregunta de un usuario */
                        contador=0;
                        do {
                          reg=buscar_ip(accounting[contador].path,ip,accounting[contador].log);
                          contador++;
                        } while((reg.tipo==STOP)&&(accounting[contador].path!=NULL));
		}

	/*
	** Hack para poder asignar un usuario fijo a una IP fija
	*/

		for(contador=0;(ip!=NULL)&&(IP_USUARIO[contador].ip!=NULL);contador++) {
		  if(!strcmp(ip,IP_USUARIO[contador].ip)) {
		    reg.tipo=START;
		    strcpy(reg.nombre,IP_USUARIO[contador].usuario);
		    break;
		  }
		}
	
		if(reg.tipo!=STOP) {	/* Lo tenemos */
		struct passwd *pwd;
		struct caller_id *telef;
		
                        p=strchr(reg.nombre,'@');
                        if(p!=NULL) *p='\0';
			if(anteriores_flag) {
				lista_antiguo(reg.nombre);
				exit(0);
			}
			envia_3();
			printf("<!mensual>\n%s<p>\n",reg.nombre);
#ifdef NOPASSWD
                        pwd=NULL;
#else
			pwd=getpwnam(reg.nombre);
#endif
			if(pwd!=NULL) {
				if(pwd->pw_gecos!=NULL)
					printf("%s<p>\n",pwd->pw_gecos);
				else
				printf("Informaci&oacute;n no disponible<p>\n");
			} else printf("(An&oacute;nimo; cuenta corporativa)<p>\n");

                        if(p!=NULL) *p='@';
			val.num_conexiones=val.tiempo=0;
			val.bytes_tx=val.bytes_rx=0;
			val.paquetes_tx=val.paquetes_rx=0;
			val.numero=NULL;

		/* IP del usuario, si est conectado ahora */
			*ip_log='\0';
	
                        if(p==NULL) p=reg.nombre+strlen(reg.nombre);
                        for(contador=0;accounting[contador].path!=NULL;contador++) {
                          strcpy(p,accounting[contador].AT_ISP);
                          analiza_conexion(accounting[contador].path,reg.nombre,accounting[contador].log);
                        }

			printf("<!mensual>\n%ld<p>\n",val.num_conexiones);
     			printf("%ld Horas, %ld Minutos y %ld Segundos<p>\n",
	        		val.tiempo/3600,(val.tiempo%3600)/60,
        			val.tiempo%60);
        		printf("%d (",causa_desconexion);
        		switch(causa_desconexion) {
        		case 0: printf("No hay conexiones previas"); break;

		/* Desconexiones ASCEND */
        		case 1010: printf("Portadora no detectada"); break;
        		case 1011:
        		case 1185: printf("P&eacute;rdida de portadora"); break;
        		case 1021: printf("TimeOut por inactividad"); break;
        		case 1042: printf("Clave erronea o demasiadas sesiones"); break;
                        case 1020:
        		case 1045: printf("Desconexi&oacute;n normal"); break;
        		case 1100: printf("Tiempo de conexi&oacute;n excedido"); break;

		/* Desconexiones estandar */
			case 1: printf("Desconexi&oacute;n normal"); break;
			case 2: printf("P&eacute;rdida de portadora"); break;
			case 3: printf("Sin Servicio"); break;
			case 4: printf("TimeOut por inactividad"); break;
			case 5: printf("Tiempo de conexi&oacute;n excedido"); break;
			case 6: printf("Reset Administrativo"); break;
			case 7: printf("Reinicio Administrativo"); break;
			case 8: printf("Error de Puerto"); break;
                        case 9: printf("Error de NAS"); break;
			case 10: printf("Solicitado por el NAS"); break;
			case 11: printf("NAS Reiniciado"); break;
			case 12: printf("Puerto no finalizado"); break;
			case 13: printf("Puerto Preempted"); break;
			case 14: printf("Puerto Suspendido"); break;
			case 15: printf("Servicio No Disponible"); break;
			case 16: printf("CallBack"); break;
			case 17: printf("User Error"); break;
			case 18: printf("Host Request"); break;

			default: printf("Raz&oacute;n desconocida. "
				"<a href=mailto:ayuda@argo.es>"
				"Comunicar a ARGO</a>."); break;
        		}
        		printf(")<br>\n%s<br>\n(%ld Horas, %ld Minutos y %ld Segundos)<p>\n",
				!ultima_conexion ? "" : ctime(&ultima_conexion),
				ultimo_tiempo/3600,(ultimo_tiempo%3600)/60,
				ultimo_tiempo%60);
			telef=val.numero;
			while(telef!=NULL) {
				if(*telef->numero=='\0')
					printf("Central Anal&oacute;gica ");
				else
					printf("%s ",telef->numero);

				printf("(%ld conexiones - %ldh, "
				"%ldmin y %ldseg)<br>\n",
					telef->num_conexiones,
					telef->tiempo/3600,
					(telef->tiempo%3600)/60,
					telef->tiempo%60);
				telef=telef->siguiente;
			}
			printf("</table><p><table width=100%%>\n"
				"<tr><td width=49%% align=right>\n"
				"Bytes enviados<br>\n"
				"Bytes recibidos<p>\n"
				"</td><td width=2%%></td>"
				"<td>\n"
				"%lld<br>%lld<p>",val.bytes_tx,val.bytes_rx
				);
			printf("</td></tr><tr><td width=49%% align=right>"
				"Paquetes IP enviados<br>\n"
				"Paquetes IP recibidos<p>\n"
				"</td><td width=2%%>"
				"</td><td>\n"
				"%lld<br>%lld<p>\n",
				val.paquetes_tx,val.paquetes_rx
				);			
			envia_2();
			printf("<!mensual>\n");

			if(*ip_log) {
				printf("<br><center><b><font size=+2>Direcci&oacute;n "
					"IP actual: %s</font></b>\n<br>(%s)</center><p>\n",
					ip_log,ctime(&fecha));
			} else {
				printf("<center><h2>Ese usuario no "
					"est&aacute; conectado en "
					"este momento"
					"</h2></center>\n");
			}
			if(string==NULL) string="";
			printf("<center><a href=/cgi-bin/conexiones?%s*>"
				"Archivo hist&oacute;rico</a></center>\n",string);
			envia_4();
		} else printf("<center><h2>No se ha podido encontrar "
				"informaci&oacute;n sobre esta "
				"conexi&oacute;n</h2></center>\n");
		
	} else printf("<center><h2>Usted no es usuario de ARGO</h2>"
		"<font size=+1><b><i>El sistema no le reconoce como un usuario "
		"de este proveedor. Si usted es realmente un abonado "
		"de ARGO, el problema es que est&aacute; accediendo "
		"a este servicio de informaci&oacute;n a trav&eacute;s "
		"de una red externa. Este servicio solo est&aacute; "
		"disponible si accede desde su conexi&oacute;n ARGO "
		"</i></b></font></center>\n");
	
	envia_post();
	
	exit(0);
}

