/*
** LEE_LOG.C
**
** Rutinas genericas de lectura de LOGS.
**
**
** (c) 1.997-99 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 23/Abr/97 Primera versin de este fichero.
**           Aadida cach de disco.
**           Aadidas llamadas para la apertura y cierre de logs
**           Si llega al final del fichero, lo cierra automticamente.
**
** 26/Jun/97 Aadidos CSIVs de Bilbao y Sevilla
**
** 18/Sep/97 Aadidos campos de trfico Entrada/Salida
**
** 28/Sep/98 Por un bug no leia el campo "Acct-Session-Id"
**
** 13/Oct/98 En vez de indicar el CSIV, se indica directamente
**           el Path del fichero de detalle.
**           Define la interfaz publica de este modulo.
**           Enorme mejora (X3) en el rendimiento, gracias a
**           - Trabajo con cache en memoria.
**           - Lectura linea a linea, no caracter a caracter.
**           Cambiamos los atoll(), que en Linux dan problemas,
**           por atol(). Esto afecta a los contadores de
**           bytes y paquetes.
**
** 20/Oct/98 Se ha segregado el modulo. Aqui solo quedan
**           las rutinas genericas de acceso a logs.
**           El uso de una libreria de cadenas propia mejora el rendimiento.
**
** 18/Nov/98 Cuando hay un error con un fichero, no imprime nada.
**           Cuando el fichero se ciella, cuida de que no se cree
**           un alias a otro FD (por ejemplo, 0=STDIN)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "lee_log.h"

#define strchr(a,b)	strchr_lee_log(a,b)
/*
#define strcmp(a,b)	strcmp_lee_log(a,b)
*/
#include "util.c"

#define LONG_BUF	CAMPO_LONG

#define LONG_CACHE	(16*1024)

static char cache[LONG_CACHE+1];	/* OJO CON CAMBIAR DE FICHEROS!!! */
static int p_cache;	/* Puntero */
static int cache_tam=0;	/* Bytes vlidos en la cach */

static int fichero=-1;

void inicializa_registro(registro_log *registro) {

  registro->tipo=STOP;

  *registro->nombre=*registro->ip=
    *registro->sesion_id=*registro->caller_id='\0';

  registro->tiempo=registro->duracion=
    registro->causa=registro->velocidad=
    registro->bytes_tx=registro->bytes_rx=
    registro->paquetes_tx=registro->paquetes_rx=0;
}

void abre_log_cache(char *f_detalle)
{
	p_cache=cache_tam=0;
	fichero=open(f_detalle,O_RDONLY);
}

void cierra_log_cache(void)
{
	close(fichero);
	fichero=-1;
}

log_estado lee_log_cache(char *linea)
{
char *p=linea,*p2;
int len;

  *linea='\0';
  while(!0) {
    if(p_cache>=cache_tam) { /* Cach agotada */
      *p='\0';
      p_cache=0;
      cache_tam=read(fichero,cache,LONG_CACHE);
      if(cache_tam>0) cache[cache_tam]='\0';
      else return LOG_EOF; /* Puede ser 0 o -1 */
    }

    p2=strchr(cache+p_cache,'\n');
    if(p2==NULL) { /* Hemos leido una linea de forma incompleta */
      len=strlen(cache+p_cache);
      if(p-linea+len>=LONG_BUF-4) return LINE_TOO_LONG;
      memcpy(p,cache+p_cache,len);
      p+=len;	
      p_cache+=len;
      continue;
    } else { /* Linea completada */
      len=p2-cache-p_cache;
      if(p-linea+len>=LONG_BUF-4) return LINE_TOO_LONG;
      if(len) memcpy(p,cache+p_cache,len); /* Necesario por longitud 0 */
      p+=len;
      p_cache+=len+1; /* Nos saltamos el '\n' */
      *p++='\0'; /* Cierra la linea leida */
      return LOG_OK;
    }
  }
}

