/*
** LEE_LOG.H
**
** Define las estructuras de datos para los LOGs.
** Rutinas genericas de lectura de LOGS.
**
**
** (c) 1.997-99 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 23/Abr/97 Primera version de este fichero
**
** 26/Jun/97 Anadidos CSIVs de Bilbao y Sevilla
**
** 18/Sep/97 Anadidos campos de trafico Entrada/Salida
**
** 13/Oct/98 La descripcion de los CSIVs se realiza a traves
**           de variables "externas".
**           Define la interfaz publica de este modulo.
**           Cambiamos los atoll(), que en Linux dan problemas,
**           por atol(). Esto afecta a los contadores de
**           bytes y paquetes.
**
** 20/Oct/98 Separa este modulo en dos. En este ahora
**           solo quedan las rutinas generales de logs.
**
** 20/Nov/98 Los campos de fecha y hora se cambian
**           por un campo de tiempo en segundos
**           desde el epoch.
*/

#ifndef LEE_LOG_H
#define LEE_LOG_H

#define CAMPO_LONG	80

typedef struct {
	enum {START,STOP} tipo;
	long tiempo;
	char nombre[CAMPO_LONG];
	char sesion_id[CAMPO_LONG];
	long duracion;
	int causa; /* A las desconexiones ASCEND se les suma 1000 */
	long velocidad;
	char caller_id[CAMPO_LONG];
	char ip[CAMPO_LONG];
	long bytes_tx;
	long bytes_rx;
	long paquetes_tx;
	long paquetes_rx;
} registro_log;

typedef enum {LOG_OK,LOG_EOF,LINE_TOO_LONG} log_estado;

/*
** Estas rutinas estan disenadas
** para ser invocadas desde librerias
** de acceso a logs.
*/

/* Inicializa los campos del registro a cero y '\0' */
void inicializa_registro(registro_log *registro);

/* Abre un fichero, con cache */
void abre_log_cache(char *f_detalle);

/* Cierra el fichero abierto */
void cierra_log_cache(void);

/* Lee una linea entera */
log_estado lee_log_cache(char * linea);

#endif

